#!/bin/sh

RRD_PATH=/var/lib/rrd/
NAV=/var/tmp/nav.inc.html

#################### interface
check_if_db() {
	max_in=$2
	max_out=$3
	if [ ! -e ${RRD_PATH}/traffic-$1.rrd ]; then
		echo "creating traffic database for $1"
		rrdtool create ${RRD_PATH}/traffic-$1.rrd \
		-s 300 \
		DS:in:COUNTER:600:0:${max_in:="U"} \
		DS:out:COUNTER:600:0:${max_out:="U"} \
		DS:pkg-in:COUNTER:600:0:U \
		DS:pkg-in-err:COUNTER:600:0:U \
		DS:pkg-in-drop:COUNTER:600:0:U \
		DS:pkg-out:COUNTER:600:0:U \
		DS:pkg-out-err:COUNTER:600:0:U \
		DS:pkg-out-drop:COUNTER:600:0:U \
		RRA:AVERAGE:0.5:1:576 \
		RRA:AVERAGE:0.5:6:672 \
		RRA:AVERAGE:0.5:24:732 \
		RRA:AVERAGE:0.5:144:146
	fi
}

update_if() {
	if=`grep $1 /proc/net/dev | awk -F "[: ]+" '{print $3 ":" $4 ":" $5 ":" $6 ":" $11 ":" $12 ":" $13 ":" $14}'`
	echo update if $1 $if
	rrdtool update ${RRD_PATH}/traffic-$1.rrd -t in:pkg-in:pkg-in-err:pkg-in-drop:out:pkg-out:pkg-out-err:pkg-out-drop N:$if
}

#################### wireless
check_iw_db() {
	if [ ! -e ${RRD_PATH}/wlan-$1.rrd ]; then
		echo "creating wireless database for $1"
		rrdtool create ${RRD_PATH}/wlan-$1.rrd \
		-s 300 \
		DS:qual:GAUGE:600:0:60 \
		DS:signal:GAUGE:600:-256:0 \
		DS:noise:GAUGE:600:-256:0 \
		DS:rate:GAUGE:600:0:100 \
		DS:disc-nwid:COUNTER:600:0:U \
		DS:disc-crypt:COUNTER:600:0:U \
		DS:disc-frag:COUNTER:600:0:U \
		DS:disc-retry:COUNTER:600:0:U \
		DS:disc-misc:COUNTER:600:0:U \
		DS:missed-beacon:COUNTER:600:0:U \
		RRA:AVERAGE:0.5:1:576 \
		RRA:AVERAGE:0.5:6:672 \
		RRA:AVERAGE:0.5:24:732 \
		RRA:AVERAGE:0.5:144:146
	fi
}

update_iw() {
	qual=`iwconfig $1|grep Quality|cut -d":" -f2|cut -d" " -f1|cut -d"/" -f1`
	signal=`iwconfig $1|grep Signal|cut -d":" -f3|cut -d" " -f1`
	noise=`iwconfig $1|grep Signal|cut -d":" -f4|cut -d" " -f1`
	rate=`iwconfig $1|grep Rate|cut -d"M" -f1|cut -d"k" -f1|cut -b20-22`
	disc=`grep $1 /proc/net/wireless | awk '{print $6 ":" $7 ":" $8 ":" $9 ":" $10 ":" $11 }'`
	echo update iw $1 $qual:$signal:$noise:$rate:$disc
	rrdtool update ${RRD_PATH}/wlan-$1.rrd -t qual:signal:noise:rate:disc-nwid:disc-crypt:disc-frag:disc-retry:disc-misc:missed-beacon N:$qual:$signal:$noise:$rate:$disc
}

#################### system
check_sys_db() {
	if [ ! -e ${RRD_PATH}/system.rrd ]; then
		echo "creating system database"
		rrdtool create ${RRD_PATH}/system.rrd \
		-s 300 \
		DS:mem-used:GAUGE:600:0:U \
		DS:mem-free:GAUGE:600:0:U \
		DS:mem-cache:GAUGE:600:0:U \
		DS:load:GAUGE:600:0:U \
		DS:user:COUNTER:600:0:U \
		DS:nice:COUNTER:600:0:U \
		DS:system:COUNTER:600:0:U \
		DS:du-root:GAUGE:600:0:U \
		DS:df-root:GAUGE:600:0:U \
		DS:du-tmp:GAUGE:600:0:U \
		DS:df-tmp:GAUGE:600:0:U \
		RRA:AVERAGE:0.5:1:576 \
		RRA:AVERAGE:0.5:6:672 \
		RRA:AVERAGE:0.5:24:732 \
		RRA:AVERAGE:0.5:144:146
	fi
}

update_sys() {
	mem=`grep "Mem:" /proc/meminfo|cut -d" " -f4,5,22|sed s/\ /:/g`
	load=`cut -d" " -f1 /proc/loadavg`
	stat=`grep cpu0 /proc/stat | cut -d" " -f2,3,4 |sed s/\ /:/g`
	dfr=`df |grep /dev/root | awk '{print $3 ":" $4}'`
	dft=`df |grep tmpfs | awk '{print $3 ":" $4}'`
	echo update system $mem:$load:$stat:$dfr:$dft
	rrdtool update ${RRD_PATH}/system.rrd -t mem-used:mem-free:mem-cache:load:user:nice:system:du-root:df-root:du-tmp:df-tmp N:$mem:$load:$stat:$dfr:$dft
}

test -d $RRD_PATH || mkdir -p $RRD_PATH

# system
check_sys_db
update_sys
echo "| <a href=\"system.rrd.html\">system</a>" > $NAV

echo "| traffic: " >> $NAV
# interfaces
for i in `awk -F : '{n++} {if (n>2 && !/wifi/ && !/lo/) print $1}' /proc/net/dev`; do
	# estimate maximum for db protection
	max=""
	[[ $i == "eth0" ]] && max="13107200 13107200" # max 100Mbps
	[[ ${i:0:3} == "ppp" ]] && max="786432 786432" # max 6Mbps
	[[ ${i:0:4} == "wlan" ]] && max="786432 786432" # max 6Mbps
	[[ ${i:0:3} == "ath" ]] && max="7864320 7864320" # max 60Mbps
	
	check_if_db $i
	update_if $i
	
	echo "<a href=\"traffic-if.rrd.html?ifname=$i\">$i</a> " >> $NAV
done

echo "| wlan: " >> $NAV
# wireless interfaces
for i in `awk -F : '{n++} {if (n>2 && !/wifi/) print $1}' /proc/net/wireless`; do
	check_iw_db $i
	update_iw $i
	
	echo "<a href=\"wlan.rrd.html?ifname=$i\">$i</a> " >> $NAV
done

echo "|" >> $NAV